;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************
;SERIAL EEPROM/DALLAS 1624 TEMP IC SUBROTINES
;
;           PIC          SERIAL EE/TEMP IC
;       ------------    -------------------
;      
;       Pin 7 (GPIO 0) -->  SDATA
;       Pin 5 (GPIO 2) -->  SCLK
;
;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

INIT_TMP_SENSOR				;SUB TO INITIATE THE TEMP SENSOR
					;1 READ ONLY
        CALL            BSTART          ;SEND START BIT
        MOVLW           B'10010000'     ;SET SLAVE ADDRESS AND WRITE MODE
        MOVWF           TXBUF           ;STORE IN TX BUFFER
        CALL            TX              ;SEND TO DS1624 
	MOVLW		H'AC'		;MOVE H'AC' TO W (ACCESS CONFIG)
        MOVWF           TXBUF           ;STORE IN TX BUFFER
        CALL            TX              ;SEND TO DS1624 

	MOVLW		B'00000001'	;MOVE H'01' TO W (1 SHOT READ)	
        MOVWF           TXBUF           ;STORE IN TX BUFFER
        CALL            TX              ;SEND TO DS1624 
        CALL            BSTOP           ;SEND STOP BIT

        MOVLW           B'11011'        ;SET SDA AND SCLK AS INPUTS
        TRIS            GPIO            ;TRISTATE GPIO

        RETLW           0               ;RETURN

;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

START_TMP_READ				;SUB TO START TEMPREATURE READ 
        CALL            BSTART          ;SEND START BIT
        MOVLW           B'10010000'     ;SET SLAVE ADDRESS AND WRITE MODE
        MOVWF           TXBUF           ;STORE IN TX BUFFER
        CALL            TX              ;SEND TO DS1624 
	
	MOVLW		H'EE'		;MOVE H'EE' TO W (START READ)
        MOVWF           TXBUF           ;STORE IN TX BUFFER
        CALL            TX              ;SEND TO DS1624 
        CALL            BSTOP           ;SEND STOP BIT

        MOVLW           B'11011'        ;SET SDA AND SCLK AS INPUTS
        TRIS            GPIO            ;TRISTATE GPIO
	
	RETURN				;RETURN

;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

READ_TEMP				;READ, AND CONVERT TEMP
        CALL            BSTART          ;SEND START BIT
        MOVLW           B'10010000'     ;SET SLAVE ADDRESS AND WRITE MODE
        MOVWF           TXBUF           ;STORE IN TX BUFFER
        CALL            TX              ;SEND TO DS1624 
	
	MOVLW		H'AA'		;MOVE H'AA' TO W (READ TEMP)
        MOVWF           TXBUF           ;STORE IN TX BUFFER
        CALL            TX              ;SEND TO DS1624 

        CALL            BSTART          ;SEND START BIT
        MOVLW           B'10010001'     ;SET SLAVE ADDRESS AND READ MODE
        MOVWF           TXBUF           ;STORE IN TX BUFFER
        CALL            TX              ;SEND TO DS1624 

	BCF		FLAG1,NACK	;SEND ACKNOWLEDGE AFTER RX
        CALL            RX              ;READ BYTE FROM 1624 (HI TEMP BYTE)
	MOVF		DATAI,W		;MOVE DATA IN TO W
	MOVWF		TEMP_1		;STORE IN TEMP 1
	
	BSF		FLAG1,NACK	;SEND NO ACKNOWLEDGE AFTER RX
        CALL            RX              ;READ BYTE FROM 1624 (HI TEMP BYTE)
	MOVF		DATAI,W		;MOVE DATA IN TO W
	MOVWF		TEMP_2		;STORE IN TEMP 2

        CALL            BSTOP           ;SEND STOP BIT

	NOP	

	BTFSC		TEMP_1,MSB	;TEMPREATURE NEGATIVE?
	GOTO		SORT_NEG	;YES, SORT NEGATIVE CALC
	
	MOVLW		D'111'		;MOVE D'111' TO W (oC)
	SUBWF		TEMP_1,W	;SUBTRACT FROM TEMP 1
	
	MOVLW		D'250'		;MOVE D'250' TO W (MAX TEMP)
	BTFSC		STATUS,C    	;RESULT POSITIVE (GREATER THAN
					;110oC) ?
	GOTO 		EXIT_TEMP_CON	;YES, EXIT TEMP CONVERT
	
	RLF		TEMP_2,F	;ROTATE TEMP 2 BIT 7 INTO CARRY	
	RLF		TEMP_1,F	;ROTATE TEMP 1 LEFT (CARRY INTO LSB)	
	
	MOVLW		D'30'		;OFFSET VALUE
	ADDWF		TEMP_1,W	;ADD TO TEMP 1
	GOTO 		EXIT_TEMP_CON	;EXIT TEMP CONVERT

SORT_NEG				;RESULT NEGATIVE
	COMF		TEMP_1,F	;FLIP TEMP 1
	INCF		TEMP_1,F	;INCREASE BY 1
	
	MOVLW		D'16'		;MOVE D'15' TO W (-oC)
	SUBWF		TEMP_1,W	;SUBTRACT FROM TEMP 1
	
	MOVLW		D'00'		;MOVE D'00' TO W (MIN TEMP)
	BTFSC		STATUS,C    	;RESULT POSITIVE (GREATER THAN
					;15oC) ?
	GOTO 		EXIT_TEMP_CON	;YES, EXIT TEMP CONVERT

	RRF		TEMP_2,F	;ROTATE TEMP 2 RIGHT
	RRF		TEMP_2,F	;ROTATE TEMP 2 RIGHT
	RRF		TEMP_2,F	;ROTATE TEMP 2 RIGHT

	MOVLW		B'00011111'	;MASK FOR TOP BITS
	ANDWF		TEMP_2,F	;CLEAR TOP BITS

	MOVLW		D'16'		;MOVE D'16' TO W
	SUBWF		TEMP_2,W	;SUBTRACT FROM TEMP 2

	BSF		TEMP_1,LSB	;SET LSB OF TEMP 1 (1/2 oC)

	BTFSS		STATUS,C    	;RESULT NEGATIVE (GREATER OR = 
					;16) ?
	
	BCF		TEMP_1,LSB	;CLEAR LSB OF TEMP 1 (1/2 oC)

	MOVLW		D'30'		;MOVE D'30' TO W (OFFSET)
	MOVWF		TEMP_2		;STORE IN TEMP 2

	MOVF		TEMP_1,W	;MOVE TEMP 1 TO W
	SUBWF		TEMP_2,F	;SUBTRACT FROM TEMP 2
	
	MOVF		TEMP_2,W	;MOVE TEMP 2 TO W
	
EXIT_TEMP_CON	
	MOVWF		NEW_TEMP	;STORE W IN NEW_TEMP

	RETURN				;RETURN

;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

;WRITE BYTE ROUTINE
;ADDR MUST BE LOADED WITH ADDRESS BEFORE CALLING
;DATAO MUST BE LOADED WITH OUTPUT DATA BEFORE CALLING

;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

WRITE_BYTE
        CALL            BSTART          ;SEND START BIT

        MOVLW           B'10010000'     ;SET SLAVE ADDRESS AND WRITE MODE
        MOVWF           TXBUF           ;STORE IN TX BUFFER
        CALL            TX              ;SEND TO DS1624 

	MOVLW		H'17'		;MOVE H'17' TO W (ACCESS MEMORY)
        MOVWF           TXBUF           ;STORE IN TX BUFFER
        CALL            TX              ;SEND TO DS1624 

        MOVF            ADDR,W          ;MOVE ADDRESS TO W
        MOVWF           TXBUF           ;STORE IN TX BUFFER
        CALL            TX              ;SEND TO DS1624 

        MOVF            DATAO,W         ;MOVE DATA TO W
        MOVWF           TXBUF           ;STORE IN TX BUFFER
        CALL            TX              ;SEND TO DS1624 
        CALL            BSTOP           ;SEND STOP BIT

        MOVLW           B'11011'        ;SET SDA AND SCLK AS INPUTS
        TRIS            GPIO            ;TRISTATE GPIO

        RETLW           0               ;RETURN

;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

;READ ROUTINE
;ADDR MUST BE LOADED WITH ADDRESS BEFORE CALLING
;DATAI HOLDS THE DATA READ FROM THE EEPROM

;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

READ_E2
        CALL            BSTART          ;SEND START BIT

        MOVLW           B'10010000'     ;SET SLAVE ADDRESS AND WRITE MODE
        MOVWF           TXBUF           ;LOAD INTO TX BUFFER
        CALL            TX              ;SEND TO DS1624 

	MOVLW		H'17'		;MOVE H'17' TO W (ACCESS MEMORY)
        MOVWF           TXBUF           ;STORE IN TX BUFFER
        CALL            TX              ;SEND TO DS1624 

        MOVF            ADDR,W          ;MOVE ADDRESS TO W
        MOVWF           TXBUF           ;LOAD INTO TX BUFFER
        CALL            TX              ;SEND TO DS1624 

        CALL            BSTART          ;SEND START BIT

        MOVLW           B'10010001'     ;SET SLAVE ADDRESS AND READ MODE
        MOVWF           TXBUF           ;LOAD INTO TX BUFFER
        CALL            TX              ;SEND TO DS1624 

	BSF		FLAG1,NACK	;SEND NO ACKNOWLEDGE
        CALL            RX              ;READ BYTE FROM 1624
        CALL            BSTOP           ;SEND STOP BIT

        MOVLW           B'11011'        ;SET SDA AS INPUT SCLK AS OUTPUT
        TRIS            GPIO            ;TRISTATE GPIO
        RETLW           0               ;RETURN


;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

;START BIT SUB
;THIS SUB SENDS A START BIT TO THE EEPROM
;(LOW GOING DATA LINE WHILE CLOCK IS HIGH)          

;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

BSTART
        BSF             GPIO,SDATA      ;SET DATA HIGH
        MOVLW           B'11010'        ;SET DATA AND CLOCK AS OUTPUTS
        TRIS            GPIO            ;TRISTATE GPIO
        BCF             GPIO,SCLK       ;CLEAR CLOCK
        BSF             GPIO,SCLK       ;SET CLOCK HIGH
        BCF             GPIO,SDATA      ;CLEAR DATA
        BCF             GPIO,SCLK       ;CLEAR CLOCK
        RETLW           0
         
;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

;STOP BIT SUB
;THIS SUB SENDS A STOP BIT TO THE EEPROM
;(HIGH GOING DATA LINE WHILE CLOCK IS HIGH)

;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

BSTOP   
        BCF             GPIO,SDATA      ;CLEAR DATA
        MOVLW           B'11010'        ;SET CLOCK AND DATA AS OUTPUTS  
        TRIS            GPIO          	;TRISTATE GPIO
        BCF             GPIO,SDATA    	;CLEAR DATA
        BSF             GPIO,SCLK     	;SET CLOCK
        BSF             GPIO,SDATA    	;SET DATA
        BCF             GPIO,SCLK     	;CLEAR CLOCK
        RETLW           0               ;RETURN

;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

;TX SUB
;THIS SUB TX'S THE BYTE IN DATAO TO THE EEPROM + ACK BIT

;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

TX
        MOVLW           D'8'            ;MOVE 8 BITS TO W
        MOVWF           COUNT           ;LOAD BIT COUNTER
TXLP
        BCF             EEPROM,DO       ;CLEAR DO (DATA OUT)
        BTFSC           TXBUF,7         ;DATA LOW?
        BSF             EEPROM,DO       ;NO, SET DO

        MOVLW           B'11010'        ;SET CLOCK AND DATA AS OUTPUTS
        TRIS            GPIO          	;TRISTATE GPIO
        BTFSS           EEPROM,DO       ;DATA HIGH?
        GOTO            BITLOW          ;NO, GOTO BITLOW
        BSF             GPIO,SDATA    	;SET DATA HIGH 
        GOTO            CLKOUT          ;GOTO CLOCKOUT
BITLOW   
        BCF             GPIO,SDATA    	;CLEAR DATA    
CLKOUT  
        BSF             GPIO,SCLK     	;SET CLOCK
        BCF             GPIO,SCLK     	;CLEAR CLOCK

        RLF             TXBUF,F         ;ROTATE TXBUF LEFT THRO CARRY
        DECFSZ          COUNT,F         ;ALL DONE?
        GOTO            TXLP            ;NO, LOOP UNTIL FINISHED

        BSF             EEPROM,DI       ;SET DATA IN HIGH        
        MOVLW           B'11011'        ;MAKE SDATA AN INPUT
        TRIS            GPIO          	;TRISTATE GPIO
        BSF             GPIO,SDATA    	;SET DATA
        BSF             GPIO,SCLK     	;SET CLOCK
        BTFSS           GPIO,SDATA    	;DATA HIGH?
        BCF             EEPROM,DI       ;NO, CLEAR DI
        BCF             GPIO,SCLK     	;CLEAR CLOCK

        RETLW           0
                        
;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

;RX SUB
;THIS SUB RX'S THE BYTE IN DATAI TO THE EEPROM + ACK BIT

;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

RX
        CLRF            DATAI           ;CLEAR DATA IN
        MOVLW           D'8'            ;MOVE 8 BITS TO W
        MOVWF           COUNT           ;LOAD BIT COUNTER
        BCF             STATUS,C        ;CLEAR CARRY
RXLP     
        RLF             DATAI,F         ;ROTATE DATAI LEFT

        BSF             EEPROM,DI       ;SET DATA IN HIGH        
        MOVLW           B'11011'        ;MAKE SDATA AN INPUT
        TRIS            GPIO          	;TRISTATE GPIO
        BSF             GPIO,SDATA    	;SET DATA
        BSF             GPIO,SCLK     	;SET CLOCK
        BTFSS           GPIO,SDATA    	;DATA HIGH?
        BCF             EEPROM,DI       ;NO, CLEAR DI
        BCF             GPIO,SCLK     	;CLEAR CLOCK

        BCF             DATAI,0         ;CLEAR BIT 0 OF DATAI
        BTFSC           EEPROM,DI       ;BIT CLEAR?
        BSF             DATAI,0         ;NO, SET BIT 0 OF DATAI
        DECFSZ          COUNT,F         ;ALL DONE?
        GOTO            RXLP            ;NO, LOOP UNTIL FINISHED
	BCF		EEPROM,DO	;CLEAR DO (ACKNOWLEDGE)
	BTFSC		FLAG1,NACK	;NO ACKNOWLEDGE WANTED?
        BSF             EEPROM,DO       ;YES, SET DO (NO ACKNOWLEDGE)

        MOVLW           B'11010'        ;SET CLOCK AND DATA AS OUTPUTS
        TRIS            GPIO          	;TRISTATE GPIO
        BTFSS           EEPROM,DO       ;DATA HIGH?
        GOTO            BITLOW1         ;NO, GOTO BITLOW
        BSF             GPIO,SDATA    	;SET DATA HIGH 
        GOTO            CLKOUT1         ;GOTO CLOCKOUT
BITLOW1
        BCF             GPIO,SDATA    	;CLEAR DATA    
CLKOUT1  
        BSF             GPIO,SCLK     	;SET CLOCK
        BCF             GPIO,SCLK     	;CLEAR CLOCK

        RETLW           0               ;RETURN

;*******************(C) M.C.J. Design (support@mcjdesign.com)*********************

